@@ -1,6 +1,36 @@
 from arkivist import Arkivist
 
-print("\nTest #1 _initialization")
+print("\nTest #0 Valid file")
+dataset = Arkivist("temp/00-valid1.json")
+
+print("\nTest #0 Valid file")
+dataset = Arkivist("temp/00-valid2.json")
+
+print("\nTest #0 Invalid files")
+try:
+    dataset = Arkivist("temp/00-invalid1.json")
+except Exception as e:
+    print(str(e))
+
+print("\nTest #0 Invalid files")
+try:
+    dataset = Arkivist("temp/00-invalid2.json")
+except Exception as e:
+    print(str(e))
+
+print("\nTest #0 Invalid files")
+try:
+    dataset = Arkivist("temp/00-invalid3.json")
+except Exception as e:
+    print(str(e))
+
+print("\nTest #0 Invalid files")
+try:
+    dataset = Arkivist("temp/00-invalid4.json")
+except Exception as e:
+    print(str(e))
+
+print("\nTest #1 Initialization")
 dataset = Arkivist("temp/01-initialization.json")
 print("Empty JSON file:", dataset)
 
@@ -47,14 +77,14 @@ print("Keys: ", list(people.keys()))
 print("Values: ", list(people.values()))
 
 ## Example #6
-print("\nTest #6 _invert Hashable key-value pairs")
+print("\nTest #6 Invert Hashable key-value pairs")
 simple = Arkivist("temp/06-invert.json")
 simple.update({0: "a"})
 simple.update({1: "b"})
 simple.update({2: "c"})
 print("Normal: ", simple.show())
 simple.invert()
-print("_invert: ", simple.show())
+print("Invert: ", simple.show())
 
 ## Example #7
 print("\nTest #7 Load (replace) and Clear old data")
@@ -80,7 +110,7 @@ people.load('{"dog": {"name": "Doggy"}}')
 print("Valid String: ", people.show())
 # flush invalid input
 people.load(1234)
-print("_invalid _input: ", people.show())
+print("Invalid input: ", people.show())
 
 ## Example #9
 print("\nTest #9 Flattener")
@@ -149,7 +179,9 @@ matches = people.where("name").exclude("a", exact=False, sensitivity=False).show
 print("\nShow all items excluding names those containing 'A/a':\n ", matches)
 
 print("\nLoop over matches excluding names containing 'A/a':")
-for key, value in people.where("name").exclude("a", exact=False, sensitivity=False).query():
+for key, value in (
+    people.where("name").exclude("a", exact=False, sensitivity=False).query()
+):
     print(" - ", key, value)
 
 ## Example #14
@@ -250,4 +282,3 @@ print(" - Show:", test.show())
 print("\nTest #21b Check unexpected clearing of JSON file (encrypted)")
 test = Arkivist("temp/21-mackaroo2.json", authfile="temp/21-mackaroo2-auth.txt")
 print(" - Show:", test.show())
-
